import asyncio
import functools
from cachetools.keys import hashkey
from cachetools import TTLCache, LRUCache


def asynccachedmethod(cache, key=None, lock=None, ignore_null=True):
    def decorator(method):
        async def wrapper(self, *args, **kwargs):
            c = cache(self)
            if c is None:
                return await method(self, *args, **kwargs)
            k = hashkey(*args, **kwargs) if not key else hashkey(key(self, *args, **kwargs))
            try:
                return c[k]
            except KeyError:
                pass  # key not found
            v = await method(self, *args, **kwargs)
            try:
                # ignore null values
                if ignore_null and not v:
                    return v
                c[k] = v
            except ValueError:
                pass  # value too large
            return v

        return functools.update_wrapper(wrapper, method)
    return decorator


def asynccachepop(cache, key=None):
    def decorator(method):
        async def wrapper(self, *args, **kwargs):
            c = cache(self)
            k = hashkey(*args, **kwargs) if not key else hashkey(key(self, *args, **kwargs))
            v = await method(self, *args, **kwargs)
            try:
                c.pop(k)
            except KeyError:
                pass  # key not found
            return v

        return functools.update_wrapper(wrapper, method)
    return decorator
