import asyncio
import platform
from pkg_resources import parse_version
import functools
import contextvars

"""backport from asyncio"""
async def to_thread(func, *args, **kwargs):
    """Asynchronously run function *func* in a separate thread.

    Any *args and **kwargs supplied for this function are directly passed
    to *func*. Also, the current :class:`contextvars.Context` is propagated,
    allowing context variables from the main thread to be accessed in the
    separate thread.

    Return a coroutine that can be awaited to get the eventual result of *func*.
    """
    loop = asyncio.get_event_loop()
    ctx = contextvars.copy_context()
    func_call = functools.partial(ctx.run, func, *args, **kwargs)
    return await loop.run_in_executor(None, func_call)

if parse_version(platform.python_version()) < parse_version('3.7'):
    asyncio.all_tasks = asyncio.Task.all_tasks
    asyncio.create_task = asyncio.ensure_future
    asyncio.current_task = asyncio.Task.current_task

if parse_version(platform.python_version()) < parse_version('3.9'):
    asyncio.to_thread = to_thread

