#!/usr/bin/python3
# -#- coding: utf-8 -#-
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:textwidth=0:
#
# Copyright (C) 2020 IoT.bzh Company
# Contact: https://www.iot.bzh/licensing
#
# Author(s): Romain Forlot <romain.forlot@iot.bzh>
#
# This file is part of the RedPesk project.
#
# $RP_BEGIN_LICENSE$
# Commercial License Usage
#  Licensees holding valid commercial IoT.bzh licenses may use this file in
#  accordance with the commercial license agreement provided with the
#  Software or, alternatively, in accordance with the terms contained in
#  a written agreement between you and The IoT.bzh Company. For licensing terms
#  and conditions see https://www.iot.bzh/terms-conditions. For further
#  information use the contact form at https://www.iot.bzh/contact.
#
# GNU General Public License Usage
#  Alternatively, this file may be used under the terms of the GNU General
#  Public license version 3. This license is as published by the Free Software
#  Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
#  of this file. Please review the following information to ensure the GNU
#  General Public License requirements will be met
#  https://www.gnu.org/licenses/gpl-3.0.html.
# $RP_END_LICENSE$
#

import re
import rpm

NATIVE_ARCH="x86_64"
CROSS_SEPARATOR="--"
core_distro = ['redpesk']

# Simple enum for different build mode
SIMPLE = 0
CROSS_SIMPLE = 1
CROSS_COMPILERS = 2
ACCEL = 3

# type of koji tags
TAGTYPE_BUILD = "build"
TAGTYPE_RELEASE = "release"
TAGTYPE_EMULATED = "emulated"


def isCore(dist):
    for d in core_distro:
        if re.search(d, dist, re.IGNORECASE):
            return True
    return False

def getNative(arch):
    return NATIVE_ARCH


def isNative(arch):
    return arch == NATIVE_ARCH


def isCross(arch):
    if re.match(r'\w+{}\w+'.format(CROSS_SEPARATOR), arch):
        return True
    else:
        return False


def getCrossArch(cross_arch):
    return f"{NATIVE_ARCH}{CROSS_SEPARATOR}{cross_arch}"


def getArchCrossNative(arch):
    return re.sub(r'{}\w+'.format(CROSS_SEPARATOR), "", arch)


def getArchCrossTarget(arch):
    return re.sub(r'\w+{}'.format(CROSS_SEPARATOR), "", arch)


def getPrjDist(tag_name):
    prj = '.'
    dist = '.'

    tag_regex = re.search(
        r'(?P<prj>^.+){}(?P<dist>.+)-(emulated|build)$'.format(CROSS_SEPARATOR), tag_name)
    if tag_regex:
        prj += tag_regex.group('prj').replace('-', '.')
        dist += tag_regex.group('dist').replace('-', '.')
        if isCore(dist):
            dist = ''
    if prj == dist:
        prj_dist = ''
    else:
        prj_dist = "%s%s" % (prj, dist)

    return prj_dist


def get_arch_from_rpmmacro(target):
    rpm.reloadConfig(target=target)
    arch = rpm.expandMacro('%{_arch}')
    rpm.reloadConfig()
    return arch

def _get_tag(tag, ttype):
    newtag = re.sub(r"-\w+$", f"-{ttype}", tag)
    if newtag == tag: # no sub
        return None
    return newtag

def get_release_tag(tag):
    return _get_tag(tag, TAGTYPE_RELEASE)

def get_build_tag(tag):
    return _get_tag(tag, TAGTYPE_BUILD)

def get_emulated_tag(tag):
    return _get_tag(tag, TAGTYPE_EMULATED)

def split_tag(tag):
    tag_split = tag.split(CROSS_SEPARATOR)
    if len(tag_split) == 2:
        slug_uuid, distro = tag_split
    else:
        slug_uuid, distro = "", tag
    return slug_uuid, distro
