import io
import os
import re
import rpm
import typing
import logging
import tempfile
import contextlib

import redpesk_utils.error
import redpesk_utils.redirectoutputs

logger = logging.getLogger('specfile')
SOURCEDIR = os.path.expanduser("~/rpmbuild/SOURCES/")

def parse(content: str, retry: int = 60, create_sources: bool = False) -> rpm.spec:
    # iotbzh macros fix
    rpm.addMacro("NativeBuildRequires", "#NativeBuildRequires ")
    rpm.addMacro("TargetBuildRequires", "#TargetBuildRequires ")
    with tempfile.NamedTemporaryFile() as tmp:
        with open(tmp.name, 'w') as spec:
            spec.write(content)
        with tempSources() as sources:
            while True:
                with io.BytesIO() as stderr:
                    try:
                        with redpesk_utils.redirectoutputs.redirect_stderr(stderr):
                            return rpm.spec(tmp.name)
                    except Exception as rpmerr:
                        stderr = stderr.getvalue().decode('utf-8')
                        if not retry:
                            raise redpesk_utils.error.SpecfileError(f"{str(rpmerr).strip()} {stderr.strip()}")
                        matchTag: re.Match = re.search(r"Unknown tag: %(.*)", stderr, re.IGNORECASE)
                        matchSource: re.Match = re.search(rf'({SOURCEDIR}.*): No such file or directory', stderr, re.IGNORECASE)
                        if matchTag:
                            rpm.addMacro(matchTag.group(1), "%nil")
                            logger.info(f"{str(retry)} - {rpm.addMacro.__name__}: {matchTag.group(1)}")
                        elif create_sources and matchSource:
                            open(matchSource.group(1), 'a').close()
                            sources.append(matchSource.group(1))
                            logger.info(f"{str(retry)} - {matchSource.group(1)}")
                        else:
                            raise redpesk_utils.error.SpecfileError(f"{str(rpmerr).strip()} {stderr.strip()}")
                        retry -= 1

@contextlib.contextmanager
def tempSources() -> None:
    os.makedirs(SOURCEDIR, exist_ok=True)
    sources: typing.List = list()
    try:
        yield sources
    finally:
        for source in sources:
            os.remove(source)